************************************************************************
*     ABAQUS USER MATERIAL SUBROUTINE                                  *
*                                                                      *
*     DIRECTIONAL DISTORTIONAL HARDENING                               *
*     ALPHA MODEL - C FIXED                                            *
*                                                                      *
*     EXPLICIT INTEGRATION                                             *
*     FORWARD EULER                                                    *
*     EXACT TANGENT STIFFNESS                                          *
*                                           l.c. Vilem Klepac 2021     *
************************************************************************
      SUBROUTINE UMAT(STRESS,STATEV,DDSDDE,SSE,SPD,SCD,
     1 RPL,DDSDDT,DRPLDE,DRPLDT,
     2 STRAN,DSTRAN,TIME,DTIME,TEMP,DTEMP,PREDEF,DPRED,CMNAME,
     3 NDI,NSHR,NTENS,NSTATV,PROPS,NPROPS,COORDS,DROT,PNEWDT,
     4 CELENT,DFGRD0,DFGRD1,NOEL,NPT,LAYER,KSPT,KSTEP,KINC)
C
      INCLUDE 'ABA_PARAM.INC'
C
      CHARACTER*80 CMNAME
      DIMENSION STRESS(NTENS),STATEV(NSTATV),
     1 DDSDDE(NTENS,NTENS),DDSDDT(NTENS),DRPLDE(NTENS),
     2 STRAN(NTENS),DSTRAN(NTENS),TIME(2),PREDEF(1),DPRED(1),
     3 PROPS(NPROPS),COORDS(3),DROT(3,3),DFGRD0(3,3),DFGRD1(3,3)
C  EXTRA VARIABLES DIMENSION
      DIMENSION DYAD(NTENS,NTENS),DYIELD(NTENS),POM(NTENS),TNR(NTENS),
     1 ALPHA(NTENS),DEV(NTENS),DYIELDALPHA(NTENS),DSTRESSTR(NTENS)
C
      COMMON /PROPSBLOCK/ COMPROPS(11)

************************************************************************
C  TOLERANCE (TOLERANCE WILL BE 'TOL' TIMES INITIAL YIELD LIMIT)
      TOL = 1.D-10
C  PLASTIC EXPLICIT ITERATIONS DIVISION
      NINT = 10
************************************************************************
      DO I=1,NPROPS
         COMPROPS(I)= PROPS(I)
      END DO

C  1st LAMME
      TE1 = PROPS(2)*PROPS(1)/(1+PROPS(2))/(1-2.D0*PROPS(2))
C  2nd LAMME
      TE2 = 0.5D0*PROPS(1)/(1+PROPS(2))
C  EFFECTIVE PLASTIC STRAIN RECOVERY

      EP = STATEV(1)
C  K RECOVERY
      IF (STATEV(2).EQ.0.D0) STATEV(2) = PROPS(3)
      EK = STATEV(2)
C  C RECOVERY
      IF (STATEV(3).EQ.0.D0) STATEV(3) = PROPS(8)
      C = STATEV(3)
C  BACKSTRESS RECOVERY

      DO I=1,NTENS
         ALPHA(I) = STATEV(I+3)
      END DO

C  TOLERANCE
      TOL = TOL*PROPS(3)
C  ELASTIC JACOBIAN
      DO I=1,NTENS
         DO J=1,NTENS
            DDSDDE(I,J) = 0.D0
         END DO
      END DO
      DO I=1,NDI
         DDSDDE(I,I) = TE1 + 2.D0*TE2
      END DO
      DO I=2,NDI
         K = I-1
         DO J=1,K
            DDSDDE(I,J) = TE1
            DDSDDE(J,I) = TE1
         END DO
      END DO
      K = NDI
      DO I=1,NSHR
         K = K+1
         DDSDDE(K,K) = TE2
      END DO
C
      CALL MAPP42(DDSDDE,DSTRAN,DSTRESSTR,NDI,NSHR)
      DO I=1,NTENS
            STRESS(I) = STRESS(I) + DSTRESSTR(I)
            IF (ABS(STRESS(I)).LT.1.D-10) THEN
               STRESS(I)=0.D0
            END IF
      END DO
      CALL YIELDCON(STRESS,ALPHA,DEV,TNR,ALPHANR,CONST,EK,C,
     1              YIELD,DYIELD,NDI,NSHR,TOL)
      CALL SCAL2(DYIELD,DSTRESSTR,VEC,NDI,NSHR)
*  PLASTIC PART  *******************************************************
      IF ((YIELD.GE.0.D0).AND.(VEC.GT.0.D0)) THEN
C
         BETA=0
         RM=YIELD/EK
         IF (RM.LT.1.D0) THEN
            M=MAX(1.D0+NINT*RM,2.D0)
         ELSE
            M=MIN(1.D0+NINT*RM,1.D6)
         END IF
         DO K=1,20
            IF (ABS(YIELD).LT.TOL) EXIT
            DBETA = YIELD/VEC
            BETA = BETA + DBETA
            DO I=1,NTENS
               STRESS(I) = STRESS(I) - DBETA*DSTRESSTR(I)
               IF (ABS(STRESS(I)).LT.1.D-12) THEN
                  STRESS(I)=0.D0
               END IF
            END DO
            CALL YIELDCON(STRESS,ALPHA,DEV,TNR,ALPHANR,CONST,EK,C,
     1                    YIELD,DYIELD,NDI,NSHR,TOL)
            CALL SCAL2(DYIELD,DSTRESSTR,VEC,NDI,NSHR)
         END DO
         DO I=1,NTENS
            DSTRESSTR(I) = BETA/M*DSTRESSTR(I)
         END DO
C
         DO I=1,M
            CALL FLOWRULE(STRESS,ALPHA,DEV,TNR,ALPHANR,EP,EK,C,
     1                    YIELD,DYIELD,DYIELDALPHA,DDSDDE,
     2                    DSTRESSTR,CONST,NDI,NSHR,TOL)
         END DO
C
         CALL ADDDERIV(STRESS,ALPHA,DEV,TNR,ALPHANR,CONST,EK,C,
     1                 YIELD,DYIELD,DYIELDALPHA,NDI,NSHR,TOL)
         CALL MAPP42(DDSDDE,DYIELD,POM,NDI,NSHR)
         CALL SCAL2(DYIELD,POM,DYCDY,NDI,NSHR)
         CALL SCAL2E(DYIELD,DYIELDALPHA,DYDYA,NDI,NSHR)
         CALL SCAL2(ALPHA,DYIELDALPHA,ADYA,NDI,NSHR)
         CALL SCAL2E(DYIELD,DYIELD,DYIELDNORM,NDI,NSHR)
         DYIELDNORM = SQRT(DYIELDNORM)
         CALL TENS22(POM,POM,DYAD,NDI,NSHR)
         TKP = 0.5D0*PROPS(4)*(1-PROPS(5)*EK)
     1         - PROPS(6)*DYDYA + PROPS(6)*PROPS(7)*ADYA*DYIELDNORM
         DYCDY = DYCDY + TKP
C
         DO I=1,NTENS
            DO J=1,NTENS
                DDSDDE(I,J) = DDSDDE(I,J) - DYAD(I,J)/DYCDY
            END DO
         END DO
      END IF
*  END PLASTIC PART  ***************************************************
C  EFFECTIVE PLASTIC STRAIN UPDATE
      STATEV(1) = EP
C  K UPDATE
      STATEV(2) = EK
C  C UPDATE
      STATEV(3) = C
C  BACKSTRESS UPDATE
      DO I=1,NTENS
         STATEV(I+3) = ALPHA(I)
      END DO
C
!      CALL YIELDCON(STRESS,ALPHA,DEV,TNR,ALPHANR,CONST,EK,C,
!     1              YIELD,DYIELD,NDI,NSHR,TOL)
!      WRITE(7,*), 'YIELD', YIELD
C
      RETURN
      END
************************************************************************
*     ABAQUS SDVINI SUBROUTINE                                         *
*                                                                      *
*     INITIAL VALUES OF STATEV FIELD                                   *
************************************************************************
      SUBROUTINE SDVINI(STATEV,COORDS,NSTATV,NCRDS,NOEL,NPT,
     1 LAYER,KSPT)
C
      INCLUDE 'ABA_PARAM.INC'
C
      DIMENSION STATEV(NSTATV),COORDS(NCRDS)
C
      DO I=1,NSTATV
         STATEV(I) = 0.D0
      END DO
      STATEV(2) = 128.D0
C
      RETURN
      END
************************************************************************
*     ABAQUS UVARM SUBROUTINE                                          *
*                                                                      *
*     PLASTIC STRAIN RECORD TO ODB                                     *
************************************************************************
      SUBROUTINE UVARM(UVAR,DIRECT,T,TIME,DTIME,CMNAME,ORNAME,
     1 NUVARM,NOEL,NPT,LAYER,KSPT,KSTEP,KINC,NDI,NSHR,COORD,
     2 JMAC,JMATYP,MATLAYO,LACCFLA)
C
      INCLUDE 'ABA_PARAM.INC'
C
      CHARACTER*80 CMNAME,ORNAME
      CHARACTER*3 FLGRAY(15)
      DIMENSION UVAR(NUVARM),DIRECT(3,3),T(3,3),TIME(2)
      DIMENSION ARRAY(15),JARRAY(15),JMAC(*),JMATYP(*),COORD(*)
      DIMENSION S(NDI+NSHR),E(NDI+NSHR)
C
      COMMON /PROPSBLOCK/ COMPROPS(11)
C
      CALL GETVRM('S',ARRAY,JARRAY,FLGRAY,JRCD,JMAC,JMATYP,
     1 MATLAYO,LACCFLA)
      DO I=1,NDI+NSHR
         S(I) = ARRAY(I)
      END DO
      CALL GETVRM('E',ARRAY,JARRAY,FLGRAY,JRCD,JMAC,JMATYP,
     1 MATLAYO,LACCFLA)
      DO I=1,NDI+NSHR
         E(I) = ARRAY(I)
      END DO
      UVAR(1) = E(1) - 1.D0/COMPROPS(1)*(S(1)-COMPROPS(2)*(S(2)+S(3)))
      UVAR(2) = E(2) - 1.D0/COMPROPS(1)*(S(2)-COMPROPS(2)*(S(1)+S(3)))
      UVAR(3) = E(3) - 1.D0/COMPROPS(1)*(S(3)-COMPROPS(2)*(S(2)+S(1)))
      UVAR(4) = E(4) - 2.D0*(1+COMPROPS(2))*S(4)/COMPROPS(1)
      UVAR(5) = E(5) - 2.D0*(1+COMPROPS(2))*S(5)/COMPROPS(1)
      UVAR(6) = E(6) - 2.D0*(1+COMPROPS(2))*S(6)/COMPROPS(1)
C
      RETURN
      END
************************************************************************
*     YIELD CONDITION                                                  *
************************************************************************
      SUBROUTINE YIELDCON(STRESS,ALPHA,DEV,TNR,ALPHANR,CONST,EK,C,
     1                    YIELD,DYIELD,NDI,NSHR,TOL)
C
      INCLUDE 'ABA_PARAM.INC'
C
      DIMENSION STRESS(NDI+NSHR),ALPHA(NDI+NSHR),TNR(NDI+NSHR),
     1 DYIELD(NDI+NSHR),DEV(NDI+NSHR)
C
      CALL DEVSTR(STRESS,DEV,NDI,NSHR)
      DO I=1,NDI+NSHR
         TNR(I) = DEV(I) - ALPHA(I)
      END DO
      CALL SCAL2NO(TNR,TNR,SANRM,NDI,NSHR)
      SANRM = SQRT(SANRM)
      DO I=1,NDI+NSHR
         IF (ABS(TNR(I)).LT.1.D-12) THEN
            TNR(I) = 0.D0
         ELSE
            TNR(I) = TNR(I)/SANRM
         END IF
      END DO
      CALL SCAL2NO(ALPHA,TNR,ALPHANR,NDI,NSHR)
      CONST=SQRT(3.D0/2.D0)/2.D0/SQRT(1-C*ALPHANR)
      DO I=1,NDI+NSHR
         DYIELD(I) = CONST*((2-C*ALPHANR)*TNR(I) - C*ALPHA(I))
      END DO
      K = NDI
      DO I=1,NSHR
         K = K+1
         DYIELD(K) = 2.D0*DYIELD(K)
      END DO
      YIELD = SQRT(3.D0/2.D0*(1-C*ALPHANR))*SANRM - EK
      IF (ABS(YIELD).LT.TOL**2) THEN
         YIELD = 0.D0
      END IF
C
      RETURN
      END
************************************************************************
*     FLOW RULE                                                        *
************************************************************************
      SUBROUTINE FLOWRULE(STRESS,ALPHA,DEV,TNR,ALPHANR,EP,EK,C,
     1                    YIELD,DYIELD,DYIELDALPHA,DDSDDE,
     2                    DSTRESSTR,CONST,NDI,NSHR,TOL)
C
      INCLUDE 'ABA_PARAM.INC'
C
      COMMON /PROPSBLOCK/ COMPROPS(11)
C
      DIMENSION STRESS(NDI+NSHR),ALPHA(NDI+NSHR),TNR(NDI+NSHR),
     1 DYIELD(NDI+NSHR),DYIELDALPHA(NDI+NSHR),DEV(NDI+NSHR),
     2 POM(NDI+NSHR),DSTRESSTR(NDI+NSHR),DDSDDE(NDI+NSHR,NDI+NSHR)
C
      CALL YIELDCON(STRESS,ALPHA,DEV,TNR,ALPHANR,CONST,EK,C,
     1              YIELD,DYIELD,NDI,NSHR,TOL)
      CALL ADDDERIV(STRESS,ALPHA,DEV,TNR,ALPHANR,CONST,EK,C,
     1              YIELD,DYIELD,DYIELDALPHA,NDI,NSHR,TOL)
C
      CALL SCAL2(DYIELD,DSTRESSTR,VEC,NDI,NSHR)
      CALL MAPP42(DDSDDE,DYIELD,POM,NDI,NSHR)
      CALL SCAL2(DYIELD,POM,DYCDY,NDI,NSHR)
      CALL SCAL2E(DYIELD,DYIELDALPHA,DYDYA,NDI,NSHR)
      CALL SCAL2(ALPHA,DYIELDALPHA,ADYA,NDI,NSHR)
      CALL SCAL2E(DYIELD,DYIELD,DYIELDNORM,NDI,NSHR)
      DYIELDNORM = SQRT(DYIELDNORM)
      TKP = 0.5D0*COMPROPS(4)*(1-COMPROPS(5)*EK)
     1    - COMPROPS(6)*DYDYA + COMPROPS(6)*COMPROPS(7)*ADYA*DYIELDNORM
      DLAMB = VEC/(DYCDY+TKP)
      EP = EP + DLAMB*SQRT(2.D0/3.D0)*DYIELDNORM
      EK = EK + 0.5D0*DLAMB*COMPROPS(4)*(1-COMPROPS(5)*EK)
      DO I=1,NDI+NSHR
         IF (I.LE.NDI) THEN
            ALPHA(I) = ALPHA(I)
     1               + DLAMB*COMPROPS(6)
     2                *(DYIELD(I)-COMPROPS(7)*DYIELDNORM*ALPHA(I))
         ELSE
            ALPHA(I) = ALPHA(I)
     1               + DLAMB*COMPROPS(6)
     2                *(0.5D0*DYIELD(I)-COMPROPS(7)*DYIELDNORM*ALPHA(I))
         END IF
      END DO
C
      DO I=1,NDI+NSHR
         STRESS(I) = STRESS(I) + DSTRESSTR(I) - DLAMB*POM(I)
         IF (ABS(STRESS(I)).LT.1.D-12) THEN
            STRESS(I)=0.D0
         END IF
      END DO
      CALL YIELDCON(STRESS,ALPHA,DEV,TNR,ALPHANR,CONST,EK,C,
     1              YIELD,DYIELD,NDI,NSHR,TOL)
C     RETURN MAPPING
      IF (YIELD.NE.0.D0) THEN
         DO N=1,20
            IF ((YIELD.GE.0.D0).AND.(YIELD.LT.TOL)) EXIT
            CALL SCAL2E(DYIELD,DYIELD,DYIELDNORM,NDI,NSHR)
            DYIELDNORM = SQRT(DYIELDNORM)
            DO I=1,NDI+NSHR
               IF (I.LE.NDI) THEN
                  STRESS(I) = STRESS(I)
     1                        - DYIELD(I)*YIELD/DYIELDNORM**2
               ELSE
                  STRESS(I) = STRESS(I)
     1                        - DYIELD(I)/2*YIELD/DYIELDNORM**2
               END IF
            END DO
            CALL YIELDCON(STRESS,ALPHA,DEV,TNR,ALPHANR,CONST,EK,C,
     1                    YIELD,DYIELD,NDI,NSHR,TOL)
         END DO
      END IF
C
      RETURN
      END
************************************************************************
*     ADDITIONAL DERIVATIVES OF YIELD FUNCTION                         *
************************************************************************
      SUBROUTINE ADDDERIV(STRESS,ALPHA,DEV,TNR,ALPHANR,CONST,EK,C,
     1                    YIELD,DYIELD,DYIELDALPHA,NDI,NSHR,TOL)
C
      INCLUDE 'ABA_PARAM.INC'
C
      DIMENSION STRESS(NDI+NSHR),ALPHA(NDI+NSHR),TNR(NDI+NSHR),
     1 DYIELD(NDI+NSHR),DYIELDALPHA(NDI+NSHR),DEV(NDI+NSHR)
C
      DO I=1,NDI+NSHR
         DYIELDALPHA(I) = CONST*((-2+C*ALPHANR)*TNR(I) + 2*C*ALPHA(I)
     1                    -C*DEV(I))
      END DO
      K = NDI
      DO I=1,NSHR
         K = K+1
         DYIELDALPHA(K) = 2.D0*DYIELDALPHA(K)
      END DO
C
      RETURN
      END
************************************************************************
*     DEVIATORIC STRESS TENSOR                                         *
************************************************************************
      SUBROUTINE DEVSTR(STRESS,DEV,NDI,NSHR)
C
      INCLUDE 'ABA_PARAM.INC'
C
      DIMENSION STRESS(NDI+NSHR),DEV(NDI+NSHR)
C
      STRMM=0.D0
      DO I=1,NDI
         STRMM = STRMM + STRESS(I)
      END DO
      DO I=1,NDI
         DEV(I) = STRESS(I) - 1.D0/3.D0*STRMM
      END DO
      I = NDI
      DO K=1,NSHR
         I = I+1
         DEV(I) = STRESS(I)
      END DO
C
      RETURN
      END
************************************************************************
*     MAPP42                                                           *
************************************************************************
      SUBROUTINE MAPP42(A,B,C,NDI,NSHR)
C
      INCLUDE 'ABA_PARAM.INC'
C
      DIMENSION A(NDI+NSHR,NDI+NSHR),B(NDI+NSHR),C(NDI+NSHR)
C
      DO I=1,NDI+NSHR
      X=0.D0
         DO J=1,NDI+NSHR
            X = X + A(I,J)*B(J)
         END DO
      C(I)=X
      END DO
C
      RETURN
      END
************************************************************************
*     SCAL2                                                            *
************************************************************************
      SUBROUTINE SCAL2(A,B,R,NDI,NSHR)
C
      INCLUDE 'ABA_PARAM.INC'
C
      DIMENSION A(NDI+NSHR),B(NDI+NSHR)
C
      R=0.D0
      DO I=1,NDI+NSHR
         R = R + A(I)*B(I)
      END DO
C
      RETURN
      END
************************************************************************
*     SCAL2E                                                           *
************************************************************************
      SUBROUTINE SCAL2E(A,B,R,NDI,NSHR)
C
      INCLUDE 'ABA_PARAM.INC'
C
      DIMENSION A(NDI+NSHR),B(NDI+NSHR)
C
      R=0.D0
      DO I=1,NDI+NSHR
         IF (I.LE.NDI) THEN
            R = R + A(I)*B(I)
         ELSE
            R = R + 0.5D0*A(I)*B(I)
         END IF
      END DO
C
      RETURN
      END
************************************************************************
*     SCAL2NO                                                          *
************************************************************************
      SUBROUTINE SCAL2NO(A,B,R,NDI,NSHR)
C
      INCLUDE 'ABA_PARAM.INC'
C
      DIMENSION A(NDI+NSHR),B(NDI+NSHR)
C
      R=0.D0
      DO I=1,NDI+NSHR
         IF (I.LE.NDI) THEN
            R = R + A(I)*B(I)
         ELSE
            R = R + 2.D0*A(I)*B(I)
         END IF
      END DO
C
      RETURN
      END
************************************************************************
*     TENS22                                                           *
************************************************************************
      SUBROUTINE TENS22(A,B,R,NDI,NSHR)
C
      INCLUDE 'ABA_PARAM.INC'
C
      DIMENSION A(NDI+NSHR),B(NDI+NSHR),R(NDI+NSHR,NDI+NSHR)
C
      DO I=1,NDI+NSHR
         DO J=1,NDI+NSHR
            R(I,J) = A(I)*B(J)
         END DO
      END DO
C
      RETURN
      END
************************************************************************
*     END OF SUBROUTINES                                               *
************************************************************************
